@@ -22,6 +22,7 @@
 #include "davcollectiondeletejob.h"
 #include "davcollectionsfetchjob.h"
 #include "davcollectionsmultifetchjob.h"
+#include "davfreebusyhandler.h"
 #include "davitemcreatejob.h"
 #include "davitemdeletejob.h"
 #include "davitemfetchjob.h"
@@ -31,13 +32,17 @@
 #include "davmanager.h"
 #include "davprotocolattribute.h"
 #include "davprotocolbase.h"
+#include "replaycache.h"
 #include "settings.h"
 #include "settingsadaptor.h"
 #include "setupwizard.h"
 
+#include <kcalcore/freebusy.h>
 #include <kcalcore/incidence.h>
 #include <kcalcore/icalformat.h>
 #include <kcalcore/todo.h>
+#include <kdatetime.h>
+#include <kjob.h>
 
 #include <akonadi/attributefactory.h>
 #include <akonadi/cachepolicy.h>
@@ -58,7 +63,7 @@ using namespace Akonadi;
 typedef QSharedPointer<KCalCore::Incidence> IncidencePtr;
 
 DavGroupwareResource::DavGroupwareResource( const QString &id )
-  : ResourceBase( id )
+  : ResourceBase( id ), FreeBusyProviderBase()
 {
   AttributeFactory::registerAttribute<EntityDisplayAttribute>();
   AttributeFactory::registerAttribute<DavProtocolAttribute>();
@@ -90,10 +95,18 @@ DavGroupwareResource::DavGroupwareResource( const QString &id )
   changeRecorder()->itemFetchScope().setAncestorRetrieval( ItemFetchScope::All );
 
   Settings::self()->setWinId( winIdForDialogs() );
+  Settings::self()->setResourceIdentifier( identifier() );
+
+  mFreeBusyHandler = new DavFreeBusyHandler( this );
+  connect( mFreeBusyHandler, SIGNAL(handlesFreeBusy(QString,bool)), this, SLOT(onHandlesFreeBusy(QString,bool)) );
+  connect( mFreeBusyHandler, SIGNAL(freeBusyRetrieved(QString,QString,bool,QString)), this, SLOT(onFreeBusyRetrieved(QString,QString,bool,QString)) );
+
+  connect( &mReplayCache, SIGNAL(etagChanged(QString,QString)), this, SLOT(onEtagChanged(QString,QString)) );
 }
 
 DavGroupwareResource::~DavGroupwareResource()
 {
+  delete mFreeBusyHandler;
 }
 
 void DavGroupwareResource::collectionRemoved( const Akonadi::Collection &collection )
@@ -114,7 +127,7 @@ void DavGroupwareResource::collectionRemoved( const Akonadi::Collection &collect
   const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( collection.remoteId() );
 
   DavCollectionDeleteJob *job = new DavCollectionDeleteJob( davUrl );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onCollectionRemovedFinished( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onCollectionRemovedFinished(KJob*)) );
   job->start();
 }
 
@@ -124,6 +137,37 @@ void DavGroupwareResource::cleanup()
   Akonadi::AgentBase::cleanup();
 }
 
+KDateTime DavGroupwareResource::lastCacheUpdate() const
+{
+  return KDateTime::currentLocalDateTime();
+}
+
+void DavGroupwareResource::canHandleFreeBusy( const QString& email ) const
+{
+  if ( !isOnline() )
+    handlesFreeBusy( email, false );
+  else
+    mFreeBusyHandler->canHandleFreeBusy( email );
+}
+
+void DavGroupwareResource::onHandlesFreeBusy( const QString &email, bool handles )
+{
+  handlesFreeBusy( email, handles );
+}
+
+void DavGroupwareResource::retrieveFreeBusy( const QString& email, const KDateTime& start, const KDateTime& end )
+{
+  if ( !isOnline() )
+    freeBusyRetrieved( email, QString(), false, i18n( "Unable to retrieve free-busy info while offline" ) );
+  else
+    mFreeBusyHandler->retrieveFreeBusy( email, start, end );
+}
+
+void DavGroupwareResource::onFreeBusyRetrieved( const QString& email, const QString& freeBusy, bool success, const QString &errorText )
+{
+  freeBusyRetrieved( email, freeBusy, success, errorText );
+}
+
 void DavGroupwareResource::configure( WId windowId )
 {
   Settings::self()->setWinId( windowId );
@@ -151,6 +195,12 @@ void DavGroupwareResource::configure( WId windowId )
 
       if ( !urls.isEmpty() )
         Settings::self()->setDisplayName( wizard.displayName() );
+
+      QString defaultUser = wizard.field( "credentialsUserName" ).toString();
+      if ( !defaultUser.isEmpty() ) {
+        Settings::self()->setDefaultUsername( defaultUser );
+        Settings::self()->setDefaultPassword( wizard.field( "credentialsPassword" ).toString() );
+      }
     }
   }
 
@@ -160,9 +210,13 @@ void DavGroupwareResource::configure( WId windowId )
   if ( windowId )
     KWindowSystem::setMainWindow( &dialog, windowId );
 
+  if ( !Settings::self()->defaultUsername().isEmpty() )
+    dialog.setPassword( Settings::self()->defaultPassword() );
+
   const int result = dialog.exec();
 
   if ( result == QDialog::Accepted ) {
+    Settings::self()->setSettingsVersion( 2 );
     Settings::self()->writeConfig();
     synchronize();
     emit configurationDialogAccepted();
@@ -186,9 +240,9 @@ void DavGroupwareResource::retrieveCollections()
   mSeenCollectionsUrls.clear();
 
   DavCollectionsMultiFetchJob *job = new DavCollectionsMultiFetchJob( Settings::self()->configuredDavUrls() );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveCollectionsFinished( KJob* ) ) );
-  connect( job, SIGNAL( collectionDiscovered( int, const QString&, const QString& ) ),
-           SLOT( onCollectionDiscovered( int, const QString&, const QString& ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onRetrieveCollectionsFinished(KJob*)) );
+  connect( job, SIGNAL(collectionDiscovered(int,QString,QString)),
+           SLOT(onCollectionDiscovered(int,QString,QString)) );
   job->start();
 }
 
@@ -219,7 +273,7 @@ void DavGroupwareResource::retrieveItems( const Akonadi::Collection &collection
 
   DavItemsListJob *job = new DavItemsListJob( davUrl );
   job->setProperty( "collection", QVariant::fromValue( collection ) );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveItemsFinished( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onRetrieveItemsFinished(KJob*)) );
   job->start();
 }
 
@@ -247,7 +301,7 @@ bool DavGroupwareResource::retrieveItem( const Akonadi::Item &item, const QSet<Q
 
   DavItemFetchJob *job = new DavItemFetchJob( davUrl, davItem );
   job->setProperty( "item", QVariant::fromValue( item ) );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveItemFinished( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onRetrieveItemFinished(KJob*)) );
   job->start();
 
   return true;
@@ -265,73 +319,36 @@ void DavGroupwareResource::itemAdded( const Akonadi::Item &item, const Akonadi::
     return;
   }
 
-  if ( mCollectionsWithTemporaryError.contains( collection.remoteId() ) ) {
-    cancelTask();
-    return;
-  }
-
-  const QString basePath = collection.remoteId();
-  if ( basePath.isEmpty() ) {
+  if ( collection.remoteId().isEmpty() ) {
     kError() << "Invalid remote id for collection " << collection.id() << " = " << collection.remoteId();
     cancelTask( i18n( "Invalid collection for item %1.", item.id() ) );
     return;
   }
 
-  KUrl url;
-  QByteArray rawData;
-  QString mimeType;
-
-  if ( item.hasPayload<KABC::Addressee>() ) {
-    const KABC::Addressee contact = item.payload<KABC::Addressee>();
-
-    const QString fileName = contact.uid();
-    if ( fileName.isEmpty() ) {
-      kError() << "Invalid contact uid";
-      cancelTask( i18n( "Client did not create a UID for item %1.", item.id() ) );
-      return;
-    }
-
-    url = KUrl( basePath + fileName + ".vcf" );
-
-    const DavProtocolAttribute *protoAttr = collection.attribute<DavProtocolAttribute>();
-    mimeType = DavManager::self()->davProtocol( DavUtils::Protocol( protoAttr->davProtocol() ) )->contactsMimeType();
-
-    KABC::VCardConverter converter;
-    rawData = converter.createVCard( contact );
-  } else if ( item.hasPayload<IncidencePtr>() ) {
-    const IncidencePtr ptr = item.payload<IncidencePtr>();
-
-    const QString fileName = ptr->uid();
-    if ( fileName.isEmpty() ) {
-      kError() << "Invalid incidence uid";
-      cancelTask( i18n( "Client did not create a UID for item %1.", item.id() ) );
-      return;
-    }
-
-    url = KUrl( basePath + fileName + ".ics" );
-    mimeType = "text/calendar";
-
-    KCalCore::ICalFormat formatter;
-    rawData = formatter.toICalString( ptr ).toUtf8();
-  } else {
+  DavItem davItem = DavUtils::createDavItem( item, collection );
+  if ( davItem.data().isEmpty() ) {
     kError() << "Item " << item.id() << " doesn't has a valid payload";
-    cancelTask( i18n( "Unable to retrieve added item %1.", item.id() ) );
+    cancelTask();
     return;
   }
 
-  const QString urlStr = url.prettyUrl();
-  kDebug() << "Item " << item.id() << " will be put to " << urlStr;
+  if ( mCollectionsWithTemporaryError.contains( collection.remoteId() ) ) {
+    mReplayCache.addReplayEntry( collection.remoteId(), ReplayCache::ItemAdded, item );
+    // We must set the remote id here. If it's changed by the server then a new item
+    // will be created and this one deleted.
+    Akonadi::Item newItem( item );
+    newItem.setRemoteId( davItem.url() );
+    changeCommitted( newItem );
+    return;
+  }
 
+  QString urlStr = davItem.url();
   const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( collection.remoteId(), urlStr );
-
-  DavItem davItem;
-  davItem.setUrl( urlStr );
-  davItem.setContentType( mimeType );
-  davItem.setData( rawData );
+  kDebug() << "Item " << item.id() << " will be put to " << urlStr;
 
   DavItemCreateJob *job = new DavItemCreateJob( davUrl, davItem );
   job->setProperty( "item", QVariant::fromValue( item ) );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onItemAddedFinished( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onItemAddedFinished(KJob*)) );
   job->start();
 }
 
@@ -346,44 +363,28 @@ void DavGroupwareResource::itemChanged( const Akonadi::Item &item, const QSet<QB
     return;
   }
 
-  if ( mCollectionsWithTemporaryError.contains( item.parentCollection().remoteId() ) ) {
+  DavItem davItem = DavUtils::createDavItem( item, item.parentCollection() );
+  if ( davItem.data().isEmpty() ) {
+    kError() << "Item " << item.id() << " doesn't has a valid payload";
     cancelTask();
     return;
   }
 
+  if ( mCollectionsWithTemporaryError.contains( item.parentCollection().remoteId() ) ) {
+    mReplayCache.addReplayEntry( item.parentCollection().remoteId(), ReplayCache::ItemChanged, item );
+    changeCommitted( item );
+    return;
+  }
+
   const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( item.parentCollection().remoteId(), item.remoteId() );
 
-  QByteArray rawData;
-  QString mimeType;
-
-  if ( item.hasPayload<KABC::Addressee>() ) {
-    const KABC::Addressee contact = item.payload<KABC::Addressee>();
-
-    KABC::VCardConverter converter;
-    rawData = converter.createVCard( contact );
-
-    mimeType = DavManager::self()->davProtocol( davUrl.protocol() )->contactsMimeType();
-  } else if ( item.hasPayload<IncidencePtr>() ) {
-    const IncidencePtr ptr = item.payload<IncidencePtr>();
-
-    KCalCore::ICalFormat formatter;
-    rawData = formatter.toICalString( ptr ).toUtf8();
-    mimeType = "text/calendar";
-  } else {
-    kError() << "Item " << item.id() << " doesn't has a valid payload";
-    cancelTask( i18n( "Unable to retrieve added item %1.", item.id() ) );
-    return;
-  }
-
-  DavItem davItem;
+  // We have to re-set the URL as it's not necessarily valid after createDavItem()
   davItem.setUrl( item.remoteId() );
   davItem.setEtag( item.remoteRevision() );
-  davItem.setContentType( mimeType );
-  davItem.setData( rawData );
 
   DavItemModifyJob *job = new DavItemModifyJob( davUrl, davItem );
   job->setProperty( "item", QVariant::fromValue( item ) );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onItemChangedFinished( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onItemChangedFinished(KJob*)) );
   job->start();
 }
 
@@ -396,7 +397,8 @@ void DavGroupwareResource::itemRemoved( const Akonadi::Item &item )
   }
 
   if ( mCollectionsWithTemporaryError.contains( item.parentCollection().remoteId() ) ) {
-    cancelTask();
+    mReplayCache.addReplayEntry( item.parentCollection().remoteId(), ReplayCache::ItemRemoved, item );
+    changeProcessed();
     return;
   }
 
@@ -407,15 +409,11 @@ void DavGroupwareResource::itemRemoved( const Akonadi::Item &item )
   davItem.setEtag( item.remoteRevision() );
 
   DavItemDeleteJob *job = new DavItemDeleteJob( davUrl, davItem );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( onItemRemovedFinished( KJob* ) ) );
+  job->setProperty( "item", QVariant::fromValue( item ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(onItemRemovedFinished(KJob*)) );
   job->start();
 }
 
-void DavGroupwareResource::aboutToQuit()
-{
-  Settings::self()->writeConfig();
-}
-
 void DavGroupwareResource::doSetOnline( bool online )
 {
   kDebug() << "Resource changed online status to" << online;
@@ -469,6 +467,9 @@ void DavGroupwareResource::onRetrieveCollectionsFinished( KJob *job )
   const DavCollection::List davCollections = fetchJob->collections();
 
   foreach ( const DavCollection &davCollection, davCollections ) {
+    if ( mReplayCache.hasReplayEntries( davCollection.url() ) )
+      mReplayCache.flush( davCollection.url() );
+
     if ( mCollectionsWithTemporaryError.contains( davCollection.url() ) ) {
       kWarning() << davCollection.url() << "is now available";
       mCollectionsWithTemporaryError.removeOne( davCollection.url() );
@@ -520,6 +521,26 @@ void DavGroupwareResource::onRetrieveCollectionsFinished( KJob *job )
     DavProtocolAttribute *protoAttr = collection.attribute<DavProtocolAttribute>( Collection::AddIfMissing );
     protoAttr->setDavProtocol( davCollection.protocol() );
 
+    DavCollection::Privileges privileges = davCollection.privileges();
+    Akonadi::Collection::Rights rights;
+
+    if ( privileges & DavCollection::All || privileges & DavCollection::Write )
+      rights |= Akonadi::Collection::AllRights;
+
+    if ( privileges & DavCollection::WriteContent )
+      rights |= Akonadi::Collection::CanChangeItem;
+
+    if ( privileges & DavCollection::Bind )
+      rights |= Akonadi::Collection::CanCreateItem;
+
+    if ( privileges & DavCollection::Unbind )
+      rights |= Akonadi::Collection::CanDeleteItem;
+
+    if ( privileges == DavCollection::Read )
+      rights |= Akonadi::Collection::ReadOnly;
+
+
+    collection.setRights( rights );
     mEtagCache.sync( collection );
     collections << collection;
   }
@@ -583,7 +604,7 @@ void DavGroupwareResource::onRetrieveItemsFinished( KJob *job )
   // to the remote server : only one request for n items instead of n requests.
   if ( protocolSupportsMultiget && !mEtagCache.changedRemoteIds().isEmpty() ) {
     DavItemsFetchJob *fetchJob = new DavItemsFetchJob( davUrl, mEtagCache.changedRemoteIds() );
-    connect( fetchJob, SIGNAL( result( KJob* ) ), this, SLOT( onMultigetFinished( KJob* ) ) );
+    connect( fetchJob, SIGNAL(result(KJob*)), this, SLOT(onMultigetFinished(KJob*)) );
     fetchJob->setProperty( "items", QVariant::fromValue( items ) );
     fetchJob->start();
 
@@ -711,23 +732,37 @@ void DavGroupwareResource::onItemFetched( KJob* job, bool isRefresh )
 
 void DavGroupwareResource::onItemAddedFinished( KJob *job )
 {
-  if ( job->error() ) {
-    cancelTask( i18n( "Unable to add item: %1", job->errorText() ) );
-    return;
-  }
-
   const DavItemCreateJob *createJob = qobject_cast<DavItemCreateJob*>( job );
-
   const DavItem davItem = createJob->item();
-
   Akonadi::Item item = createJob->property( "item" ).value<Akonadi::Item>();
   item.setRemoteId( davItem.url() );
 
+  if ( createJob->error() ) {
+    kError() << "Error when uploading item:" << createJob->error() << createJob->errorString();
+    int errorCode = createJob->error();
+    if ( errorCode > KJob::UserDefinedError )
+      errorCode -= KJob::UserDefinedError;
+
+    // We only bail out for errors that are unlikely to be recoverable
+    if ( !DavUtils::httpRequestRetryable( errorCode ) ) {
+      cancelTask( i18n( "Unable to add item: %1", job->errorString() ) );
+    }
+    else {
+      mReplayCache.addReplayEntry( item.parentCollection().remoteId(), ReplayCache::ItemAdded, item );
+      // We must set the remote id here. If it's changed by the server then a new item
+      // will be created and this one deleted.
+      Akonadi::Item newItem( item );
+      newItem.setRemoteId( davItem.url() );
+      changeCommitted( newItem );
+    }
+    return;
+  }
+
   if ( davItem.etag().isEmpty() ) {
     const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( item.parentCollection().remoteId(), item.remoteId() );
     DavItemFetchJob *fetchJob = new DavItemFetchJob( davUrl, davItem );
     fetchJob->setProperty( "item", QVariant::fromValue( item ) );
-    connect( fetchJob, SIGNAL( result( KJob* ) ), SLOT( onItemRefreshed( KJob* ) ) );
+    connect( fetchJob, SIGNAL(result(KJob*)), SLOT(onItemRefreshed(KJob*)) );
     fetchJob->start();
   } else {
     item.setRemoteRevision( davItem.etag() );
@@ -738,22 +773,33 @@ void DavGroupwareResource::onItemAddedFinished( KJob *job )
 
 void DavGroupwareResource::onItemChangedFinished( KJob *job )
 {
-  if ( job->error() ) {
-    cancelTask( i18n( "Unable to change item: %1", job->errorText() ) );
+  const DavItemModifyJob *modifyJob = qobject_cast<DavItemModifyJob*>( job );
+  const DavItem davItem = modifyJob->item();
+  Akonadi::Item item = modifyJob->property( "item" ).value<Akonadi::Item>();
+
+  if ( modifyJob->error() ) {
+    kError() << "Error when uploading item:" << modifyJob->error() << modifyJob->errorString();
+    int errorCode = modifyJob->error();
+    if ( errorCode > KJob::UserDefinedError )
+      errorCode -= KJob::UserDefinedError;
+
+    // We only bail out for errors that are unlikely to be recoverable
+    if ( !DavUtils::httpRequestRetryable( errorCode ) ) {
+      cancelTask( i18n( "Unable to change item: %1", job->errorString() ) );
+    }
+    else {
+      mReplayCache.addReplayEntry( item.parentCollection().remoteId(), ReplayCache::ItemChanged, item );
+      Akonadi::Item newItem( item );
+      changeCommitted( newItem );
+    }
     return;
   }
 
-  const DavItemModifyJob *modifyJob = qobject_cast<DavItemModifyJob*>( job );
-
-  const DavItem davItem = modifyJob->item();
-
-  Akonadi::Item item = modifyJob->property( "item" ).value<Akonadi::Item>();
-
   if ( davItem.etag().isEmpty() ) {
     const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( item.parentCollection().remoteId(), item.remoteId() );
     DavItemFetchJob *fetchJob = new DavItemFetchJob( davUrl, davItem );
     fetchJob->setProperty( "item", QVariant::fromValue( item ) );
-    connect( fetchJob, SIGNAL( result( KJob* ) ), SLOT( onItemRefreshed( KJob* ) ) );
+    connect( fetchJob, SIGNAL(result(KJob*)), SLOT(onItemRefreshed(KJob*)) );
     fetchJob->start();
   } else {
     item.setRemoteRevision( davItem.etag() );
@@ -765,8 +811,19 @@ void DavGroupwareResource::onItemChangedFinished( KJob *job )
 void DavGroupwareResource::onItemRemovedFinished( KJob *job )
 {
   if ( job->error() ) {
-    cancelTask( i18n( "Unable to remove item: %1", job->errorText() ) );
-    return;
+    kError() << "Error when deleting item:" << job->error() << job->errorString();
+    Akonadi::Item item = job->property( "item" ).value<Akonadi::Item>();
+    int errorCode = job->error();
+    if ( errorCode > KJob::UserDefinedError )
+      errorCode -= KJob::UserDefinedError;
+
+    if ( !DavUtils::httpRequestRetryable( errorCode ) ) {
+      cancelTask( i18n( "Unable to remove item: %1", job->errorString() ) );
+      return;
+    }
+    else {
+      mReplayCache.addReplayEntry( item.parentCollection().remoteId(), ReplayCache::ItemRemoved, item );
+    }
   }
 
   changeProcessed();
@@ -777,6 +834,11 @@ void DavGroupwareResource::onCollectionDiscovered( int protocol, const QString &
   Settings::self()->addCollectionUrlMapping( DavUtils::Protocol( protocol ), collection, config );
 }
 
+void DavGroupwareResource::onEtagChanged(const QString& itemUrl, const QString& etag)
+{
+  mEtagCache.setEtag( itemUrl, etag );
+}
+
 bool DavGroupwareResource::configurationIsValid()
 {
   if ( Settings::self()->remoteUrls().empty() )
